/**
 * Test program za hash funkcije i equals funkciju
 * 
 * Napravljen je da prikazuje rezultate ubacivanja istog fajla u tabele
 * razlicitih velicina.
 * 
 * Takodje se lako moze promeniti tip podataka koji se ubacuje promenama dve
 * promenljive na pocetku klase. Medjutim da bi to radilo tipovi treba da
 * prosiruju klasu InfoTip i da imaju ili prazan konstruktor ili da se nekako
 * drugacije inicijalizuje objekat `element` u ovoj klasi.
 */
public class TestHash {

	// treba promeniti tip ovog objekta i folder u kome su adekvatni
	// podaci da bi se radilo sa drugim podacima
	private InfoTip element = new Kancelarija();
	private String folder = "kancelarije";

	// Na primer ovako treba podesiti za Gadjanje mete
	// private InfoTip element = new Gadjanje();
	// private String folder = "mete";

	// a ovako za XO
	// private InfoTip element = new XO();
	// private String folder = "xo";

	/*
	 * Pravi praznu mapu i ispisuje je
	 */
	public void emptyTest() {
		OHashSet<Object> hash = new OHashSet<>();
		hash.printStats();
	}

	/*
	 * ucitava podatke iz datog fajla u tabelu date velicine, pri cemu su podaci
	 * datog tipa, te ispisuje na kraju statistike o tabeli.
	 */
	public void infoTest(String fajl, int size, InfoTip i) {
		OHashSet<InfoTip> hash = new OHashSet<>(size);
		if (Svetovid.testIn(fajl)) {
			while (Svetovid.in(fajl).hasMore()) {
				hash.add(i.ucitaj(Svetovid.in(fajl)));
			}
			Svetovid.closeIn(fajl);
		}
		hash.printStats();
	}

	/*
	 * Testira ubacivanje i izbacivanje elemenata iz tabele, cime se dodatno
	 * proverava da li hash/equals rade kako treba
	 */
	public void addRemoveTest(String fajl, int size, InfoTip i) {
		OHashSet<InfoTip> hash = new OHashSet<>(size);
		if (Svetovid.testIn(fajl)) {
			while (Svetovid.in(fajl).hasMore()) {
				hash.add(i.ucitaj(Svetovid.in(fajl)));
			}
			Svetovid.closeIn(fajl);
		}
		hash.printStats();
		InfoTip it = hash.someElement();
		if (!hash.add(it)) {
			System.out.println("success: not added again");
		} else {
			System.out.println("fail: added again!");
		}
		if (hash.remove(it)) {
			System.out.println("success: removed");
		} else {
			System.out.println("fail: not removed!");
		}
		if (hash.add(it)) {
			System.out.println("success: added back in");
		} else {
			System.out.println("fail: not added back in");
		}
	}

	public void run() {
		// emptyTest();

		String fajl = "/t2.txt";

		infoTest(folder + fajl, 101, element);
		Svetovid.out.println();
		infoTest(folder + fajl, 503, element);
		Svetovid.out.println();
		infoTest(folder + fajl, 997, element);
		Svetovid.out.println();

		// add remove radimo na manjem fajlu
		addRemoveTest(folder + "/t0.txt", 101, element);

	}

	public static void main(String[] args) {
		new TestHash().run();
	}
}
